@@ -1,10 +1,14 @@
 # ABSTRACT: get novel / bbs content from website
 package  Novel::Robot::Parser;
+
+use strict;
+use warnings;
+
 use Novel::Robot::Browser;
 use URI;
 use Encode;
 
-our $VERSION    = 0.23;
+our $VERSION    = 0.24;
 
 our %NULL_INDEX = (
     url          => '',
@@ -54,21 +58,27 @@ sub detect_site {
 
     my $site =
         ( $url =~ m#^\Qhttp://www.jjwxc.net/# )   ? 'jjwxc'
-      : ( $url =~ m#^\Qhttp://www.dddbbb.net/# )  ? 'dddbbb'
-      : ( $url =~ m#^\Qhttp://www.shunong.com/# ) ? 'shunong'
-      : ( $url =~ m#^\Qhttp://book.kanunu.org/# ) ? 'kanunu'
-      : ( $url =~ m#^\Qhttp://www.kanunu8.com/# ) ? 'kanunu'
       : ( $url =~ m#^\Qhttp://www.23hh.com/# )    ? 'asxs'
       : ( $url =~ m#^\Qhttp://www.day66.com/# ) ? 'day66'
+      : ( $url =~ m#^\Qhttp://www.dddbbb.net/# )  ? 'dddbbb'
+      : ( $url =~ m#^\Qhttp://www.23wx.com/# )    ? 'dingdian'
+      : ( $url =~ m#^\Qhttp://www.hkslg.com/# )    ? 'hkslg'
+      : ( $url =~ m#^\Qhttp://book.kanunu.org/# ) ? 'kanunu'
+      : ( $url =~ m#^\Qhttp://www.kanunu8.com/# ) ? 'kanunu'
       : ( $url =~ m#^\Qhttp://www.1kanshu.com/# ) ? 'kanshu'
       : ( $url =~ m#^\Qhttp://www.luoqiu.com/# )  ? 'luoqiu'
-      : ( $url =~ m#^\Qhttp://www.23wx.com/# )    ? 'dingdian'
+      : ( $url =~ m#^\Qhttp://www.my285.com/# )    ? 'my285'
       : ( $url =~ m#^\Qhttp://read.qidian.com/# ) ? 'qidian'
+      : ( $url =~ m#^\Qhttp://www.qqxs.cc/# ) ? 'qqxs'
+      : ( $url =~ m#^\Qhttp://www.shunong.com/# ) ? 'shunong'
       : ( $url =~ m#^\Qhttp://www.snwx.com/# )    ? 'snwx'
-      : ( $url =~ m#^\Qhttp://www.hkslg.com/# )    ? 'hkslg'
       : ( $url =~ m#^\Qhttp://www.tadu.com/# )    ? 'tadu'
-      : ( $url =~ m#^\Qhttp://www.zilang.net/# )    ? 'zilang'
+      : ( $url =~ m#^\Qhttp://www.ttzw.com/# )    ? 'ttzw'
       : ( $url =~ m#^\Qhttp://www.yanqingji.com/# )    ? 'yanqingji'
+      : ( $url =~ m#^\Qhttp://www.ybdu.com/# )    ? 'ybdu'
+      : ( $url =~ m#^\Qhttp://www.yqhhy.cc/# )    ? 'yqhhy'
+      : ( $url =~ m#^\Qhttp://www.zilang.net/# )    ? 'zilang'
+      : ( $url =~ m#^\Qhttp://ncs.xvna.com/# )    ? 'xvna'
       : ( $url =~ m#^\Qhttp://bbs.jjwxc.net/# )   ? 'hjj'
       : ( $url =~ m#^\Qhttp://tieba.baidu.com/# ) ? 'tieba'
       :                                             'unknown';
@@ -105,6 +115,8 @@ sub get_novel_ref {
         no_auto_request_url => 1,
     );
 
+    $self->update_floor_list( $r, %o );
+
     return $r;
 }
 
@@ -126,7 +138,7 @@ sub get_index_ref {
     $self->format_hashref_string( $r, $_ ) for qw/writer book/;
     $r->{chapter_num}  = $self->update_url_list($r->{chapter_list}, $r->{url});
 
-    $r->{writer_url} = $self->format_abs_url( $r->{writer_url}, $base_url );
+    $r->{writer_url} = $self->format_abs_url( $r->{writer_url}, $self->base_url );
     $r->{writer}=~s/[[:punct:]]//sg;
     $r->{book}=~s/[[:punct:]]//sg;
 
@@ -276,20 +288,32 @@ sub is_list_overflow {
 sub select_list_range {
     my ( $self, $src, $s_min, $s_max ) = @_;
 
+    my $have_id;
+    {
+        my $ref  = ref($src->[0]);
+        $have_id = ($ref and $ref eq 'HASH' and exists $src->[0]{id});
+    }
+
+    my $default_sub = sub {
+        my ($hashref, $fallback) = @_;
+        return $fallback unless $have_id;
+        return $hashref->{id} // $fallback;
+    };
+
     my $id_sub = sub {
         my ( $id, $default ) = @_;
-        return $id=~/\S/ ? $id : $default if ( exists $src->[0]{id} );
+        return ($id and $id=~/\S/) ? $id : $default if $have_id;
         return ( $id - 1 ) if ( $id and $id =~ /^\d+$/ );
         return $default;
     };
 
-    my $min = $id_sub->( $s_min, $src->[0]{id} // 0 );
-    my $max = $id_sub->( $s_max, $src->[-1]{id} // $#$src );
+    my $min = $id_sub->( $s_min, $default_sub->($src->[0], 0) );
+    my $max = $id_sub->( $s_max, $default_sub->($src->[-1], $#$src) );
 
     my @chap_list =
       map { $src->[$_] }
       grep {
-        my $j = $src->[$_]{id} // $_;
+        my $j = $have_id ? ($src->[$_]{id} // $_) : $_;
         $j >= $min and $j <= $max
       } ( 0 .. $#$src );
 
@@ -309,6 +333,14 @@ sub update_floor_list {
     $flist = [ grep { $_->{writer} eq $r->{writer} } @$flist ]
       if ( $o{only_poster} );
 
+    $flist = [ grep { $_->{content}=~/$o{grep_content}/s } @$flist ]
+      if ( $o{grep_content} );
+
+    $flist = [ grep { $_->{content}!~/$o{filter_content}/s } @$flist ]
+      if ( $o{filter_content} );
+
+    $flist->[$_]{id} ||= $_+1 for (0 .. $#$flist);
+
     $r->{floor_list} = $flist;
 
     return $self;
@@ -387,7 +419,7 @@ sub get_inner_html {
 sub format_abs_url {
     my ( $self, $chap, $base_url ) = @_;
     $base_url ||= $self->base_url();
-    return $chap if( ! $chap or $base_url!~/^http/ );
+    return $chap if( ! $chap or ! $base_url or $base_url!~/^http/ );
 
     if(ref($chap) eq 'HASH'){
         $chap->{url} = URI->new_abs( $chap->{url}, $base_url )->as_string;
